"""
Utilities
"""

# Django
from django.conf import settings
from django.utils.functional import lazy
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA SRP
from aasrp import __title__

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


def clean_setting(
    name: str,
    default_value: object,
    min_value: int = None,
    max_value: int = None,
    required_type: type = None,
):
    """
    Cleans the input for a custom setting

    Will use `default_value` if settings does not exit or has the wrong type
    or is outside define boundaries (for int only)

    Need to define `required_type` if `default_value` is `None`

    Will assume `min_value` of 0 for int (can be overridden)

    Returns cleaned value for setting
    """

    if default_value is None and not required_type:
        raise ValueError(_("You must specify a required_type for None defaults"))

    if not required_type:
        required_type = type(default_value)

    if min_value is None and required_type == int:
        min_value = 0

    if not hasattr(settings, name):
        cleaned_value = default_value
    else:
        if (
            isinstance(getattr(settings, name), required_type)
            and (min_value is None or getattr(settings, name) >= min_value)
            and (max_value is None or getattr(settings, name) <= max_value)
        ):
            cleaned_value = getattr(settings, name)
        else:
            logger.warning(
                f"You setting for {name} is not valid. Please correct it. "
                f"Using default for now: {default_value}"
            )
            cleaned_value = default_value

    return cleaned_value


# Format for output of datetime for this app
DATETIME_FORMAT = "%Y-%m-%d %H:%M"

format_html_lazy = lazy(format_html, str)



# Generated by Django 3.1.4 on 2020-12-28 13:30

# Django
import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

# AA SRP
import aasrp.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("eveonline", "0012_index_additions"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="AaSrp",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "verbose_name": "AA-SRP",
                "permissions": (
                    ("basic_access", "Can access the AA-SRP module"),
                    ("create_srp", "Can create new SRP links"),
                    ("manage_srp", "Can manage SRP"),
                    ("manage_srp_requests", "Can manage SRP requests"),
                ),
                "managed": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AaSrpLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("srp_name", models.CharField(default="", max_length=254)),
                (
                    "srp_status",
                    models.CharField(
                        choices=[
                            ("Active", "Active"),
                            ("Closed", "Closed"),
                            ("Completed", "Completed"),
                        ],
                        default="Active",
                        max_length=9,
                    ),
                ),
                ("srp_code", models.CharField(default="", max_length=16)),
                ("fleet_doctrine", models.CharField(default="", max_length=254)),
                ("fleet_time", models.DateTimeField()),
                ("aar_link", models.CharField(blank=True, default="", max_length=254)),
                (
                    "creator",
                    models.ForeignKey(
                        help_text="Who created the SRP link?",
                        null=True,
                        on_delete=models.SET(aasrp.models.get_sentinel_user),
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "fleet_commander",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="eveonline.evecharacter",
                    ),
                ),
            ],
            options={
                "verbose_name": "SRP Link",
                "verbose_name_plural": "SRP Links",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AaSrpRequest",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("request_code", models.CharField(default="", max_length=254)),
                ("ship_name", models.CharField(default="", max_length=254)),
                ("killboard_link", models.CharField(default="", max_length=254)),
                ("additional_info", models.TextField(blank=True, default="")),
                (
                    "request_status",
                    models.CharField(
                        choices=[
                            ("Pending", "Pending"),
                            ("Approved", "Approved"),
                            ("Rejected", "Rejected"),
                        ],
                        default="Pending",
                        max_length=8,
                    ),
                ),
                ("payout_amount", models.BigIntegerField(default=0)),
                ("loss_amount", models.BigIntegerField(default=0)),
                ("post_time", models.DateTimeField(default=django.utils.timezone.now)),
                (
                    "character",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="eveonline.evecharacter",
                    ),
                ),
                (
                    "creator",
                    models.ForeignKey(
                        help_text="Who created the SRP link?",
                        null=True,
                        on_delete=models.SET(aasrp.models.get_sentinel_user),
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "srp_link",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="aasrp.aasrplink",
                    ),
                ),
            ],
            options={
                "verbose_name": "SRP Request",
                "verbose_name_plural": "SRP Requests",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AaSrpRequestComment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("comment", models.TextField(blank=True, null=True)),
                (
                    "comment_type",
                    models.CharField(
                        choices=[
                            ("Comment", "SRP Request Comment"),
                            (
                                "Request Information",
                                "SRP Request Additional Information",
                            ),
                            ("Reject Reason", "SRP Reject Reason"),
                        ],
                        default="Comment",
                        max_length=19,
                    ),
                ),
                (
                    "creator",
                    models.ForeignKey(
                        on_delete=models.SET(aasrp.models.get_sentinel_user),
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "srp_request",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="aasrp.aasrprequest",
                    ),
                ),
            ],
            options={
                "verbose_name": "SRP Request Comment",
                "verbose_name_plural": "SRP Request Comments",
                "default_permissions": (),
            },
        ),
    ]



"""
Test checks for installed modules we might use
"""

# Django
from django.test import TestCase, modify_settings

# AA SRP
from aasrp.app_settings import (
    aa_discordnotify_installed,
    allianceauth_discordbot_installed,
)


class TestModulesInstalled(TestCase):
    """
    Test for installed modules
    """

    @modify_settings(INSTALLED_APPS={"remove": "aadiscordbot"})
    def test_allianceauth_discordbot_installed_should_return_false(self):
        """
        Test allianceauth_discordbot_installed should return False
        :return:
        """

        self.assertFalse(allianceauth_discordbot_installed())

    @modify_settings(INSTALLED_APPS={"append": "aadiscordbot"})
    def test_allianceauth_discordbot_installed_should_return_true(self):
        """
        Test allianceauth_discordbot_installed should return True
        :return:
        """

        self.assertTrue(allianceauth_discordbot_installed())

    @modify_settings(INSTALLED_APPS={"remove": "discordnotify"})
    def test_aa_discordnotify_installed_should_return_false(self):
        """
        Test aa_discordnotify_installed should return False
        :return:
        """

        self.assertFalse(aa_discordnotify_installed())

    @modify_settings(INSTALLED_APPS={"append": "discordnotify"})
    def test_aa_discordnotify_installed_should_return_true(self):
        """
        Test aa_discordnotify_installed should return True
        :return:
        """

        self.assertTrue(aa_discordnotify_installed())



"""
App config
"""

# Django
from django.apps import AppConfig

# AA SRP
from aasrp import __version__


class AaSrpConfig(AppConfig):
    """
    Application config
    """

    name = "aasrp"
    label = "aasrp"
    verbose_name = f"AA Ship Replacement v{__version__}"



"""
Tests for the template tags
"""

# Django
from django.template import Context, Template
from django.test import TestCase

# Alliance Auth
from allianceauth.tests.auth_utils import AuthUtils

# AA SRP
from aasrp import __version__
from aasrp.models import get_sentinel_user
from aasrp.tests.utils import create_fake_user


class TestMainCharacterName(TestCase):
    """
    Tests for main_character_name template tag
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.template = Template("{% load aasrp_user %}{{ user|main_character_name }}")

    def test_should_contain_character_name_for_users_with_main(self):
        """
        Should contain character name for user with main set
        :return:
        """

        # given
        user = create_fake_user(1001, "Bruce Wayne")
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "Bruce Wayne")

    def test_should_contain_user_character_name_for_users_without_main(self):
        """
        Should return username for users without a main character
        :return:
        """

        # given
        user = AuthUtils.create_user("john")
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "john")

    def test_should_return_deleted_user_for_sentinel_user(self):
        """
        Should return "deleted" for sentinel user
        :return:
        """

        # given
        user = get_sentinel_user()
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "deleted")

    def test_should_be_empty_for_none(self):
        """
        Test should be empty
        :return:
        """

        # given
        context = Context({"user": None})
        # when
        result = self.template.render(context)
        # then
        self.assertEqual(result, "")


class TestMainCharacterId(TestCase):
    """
    Tests for main_character_id template tag
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.template = Template("{% load aasrp_user %}{{ user|main_character_id }}")

    def test_should_contain_character_id_for_users_with_main(self):
        """
        Test should contain main character ID for users with main
        :return:
        """

        # given
        user = create_fake_user(1001, "Bruce Wayne")
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "1001")

    def test_should_contain_dummy_id_for_users_without_main(self):
        """
        Test should contain dummy ID (1) for users without main character
        :return:
        """

        # given
        user = AuthUtils.create_user("john")
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "1")

    def test_should_be_dummy_id_for_sentinel_user(self):
        """
        Test should return dummy ID (1) for sentinel user
        :return:
        """

        # given
        user = get_sentinel_user()
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "1")

    def test_should_be_dummy_id_for_none(self):
        """
        Test should return dummy ID (1) for None
        :return:
        """

        # given
        context = Context({"user": None})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "1")


class TestMainCharacterCorporationName(TestCase):
    """
    Tests for main_character_corporation_name template tag
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.template = Template(
            "{% load aasrp_user %}{{ user|main_character_corporation_name }}"
        )

    def test_should_contain_corp_name_for_users_with_main(self):
        """
        Test shoud return corporation name for users with main character
        :return:
        """

        # given
        user = create_fake_user(1001, "Bruce Wayne", 2001, "Wayne Tech Inc.", "WYT")
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "Wayne Tech Inc.")

    def test_should_be_empty_for_users_without_main(self):
        """
        Test should be empty for users without main character
        :return:
        """

        # given
        user = AuthUtils.create_user("john")
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "")

    def test_should_be_empty_for_sentinel_user(self):
        """
        Test should be empty for sentinel user
        :return:
        """

        # given
        user = get_sentinel_user()
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "")

    def test_should_be_empty_for_none(self):
        """
        Test should be empty for None
        :return:
        """

        # given
        context = Context({"user": None})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "")


class TestMainCorporationId(TestCase):
    """
    Tests for main_character_corporation_id template tag
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.template = Template(
            "{% load aasrp_user %}{{ user|main_character_corporation_id }}"
        )

    def test_should_contain_corporation_id_for_users_with_main(self):
        """
        Test should return main character corp ID for users with main character
        :return:
        """

        # given
        user = create_fake_user(1001, "Bruce Wayne", 2001, "Wayne Tech Inc.", "WYT")
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "2001")

    def test_should_be_dummy_id_for_users_without_main(self):
        """
        Test should return dummy ID (1) for users without main character
        :return:
        """

        # given
        user = AuthUtils.create_user("john")
        context = Context({"user": user})

        # when
        result = self.template.render(context)
        # then

        self.assertEqual(result, "1")

    def test_should_be_dummy_id_for_sentinel_user(self):
        """
        Test should return dummy ID (1) for sentinel user
        :return:
        """

        # given
        user = get_sentinel_user()
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "1")

    def test_should_be_dummy_id_for_none(self):
        """
        Test should return dummy ID (1) for None
        :return:
        """

        # given
        context = Context({"user": None})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "1")


class TestMainCharacterAllianceName(TestCase):
    """
    Tests for main_character_alliance_name template tag
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.template = Template(
            "{% load aasrp_user %}{{ user|main_character_alliance_name }}"
        )

    def test_should_contain_alliance_name_for_users_with_main(self):
        """
        Test should return main character alliance name for users with min character
        :return:
        """

        # given
        user = create_fake_user(
            1001,
            "Bruce Wayne",
            2001,
            "Wayne Tech Inc.",
            "WYT",
            alliance_id=3001,
            alliance_name="Wayne Enterprices",
        )

        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "Wayne Enterprices")

    def test_should_be_empty_for_users_without_main(self):
        """
        Test should be empty for users without main character
        :return:
        """

        # given
        user = AuthUtils.create_user("john")
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "")

    def test_should_be_empty_when_main_is_not_in_an_alliance(self):
        """
        Test should be empty when main character is not in an alliance
        :return:
        """

        # given
        user = create_fake_user(
            2012, "William Riker", 2012, "Starfleet", "SF", alliance_id=None
        )

        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "")

    def test_should_be_empty_for_sentinel_user(self):
        """
        Test should be empty for sentinel user
        :return:
        """

        # given
        user = get_sentinel_user()
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "")

    def test_should_be_empty_for_none(self):
        """
        Test should be empty for None
        :return:
        """

        # given
        context = Context({"user": None})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "")


class TestMainAllianceId(TestCase):
    """
    Tests for main_character_alliance_id template tag
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.template = Template(
            "{% load aasrp_user %}{{ user|main_character_alliance_id }}"
        )

    def test_should_contain_alliance_id_for_users_with_main(self):
        """
        Test should return main character alliance ID for user with main character
        :return:
        """

        # given
        user = create_fake_user(
            1001,
            "Bruce Wayne",
            2001,
            "Wayne Tech Inc.",
            "WYT",
            alliance_id=3001,
            alliance_name="Wayne Enterprises",
        )
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "3001")

    def test_should_be_dummy_id_for_users_without_main(self):
        """
        Test should return dummy ID (1) for user without main character
        :return:
        """

        # given
        user = AuthUtils.create_user("john")
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "1")

    def test_should_dummy_id_when_main_is_not_in_an_alliance(self):
        """
        Test should dummy ID (1) when main character is not in an alliance
        :return:
        """

        # given
        user = create_fake_user(
            2012, "William Riker", 2012, "Starfleet", "SF", alliance_id=None
        )

        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "1")

    def test_should_be_dummy_id_for_sentinel_user(self):
        """
        Test should return dummy ID (1) for sentinel user
        :return:
        """

        # given
        user = get_sentinel_user()
        context = Context({"user": user})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "1")

    def test_should_be_dummy_id_for_none(self):
        """
        test should return dummy ID (1) for None
        :return:
        """

        # given
        context = Context({"user": None})

        # when
        result = self.template.render(context)

        # then
        self.assertEqual(result, "1")


class TestForumVersionedStatic(TestCase):
    """
    Tests for aasrp_static template tag
    """

    def test_versioned_static(self):
        """
        Test should return static URL string with version
        :return:
        """

        context = Context({"version": __version__})
        template_to_render = Template(
            "{% load aasrp_versioned_static %}"
            "{% aasrp_static 'aasrp/css/aa-srp.min.css' %}"
        )

        rendered_template = template_to_render.render(context)

        self.assertInHTML(
            f'/static/aasrp/css/aa-srp.min.css?v={context["version"]}',
            rendered_template,
        )



# Generated by Django 3.2.6 on 2021-08-18 13:14

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveuniverse", "0005_type_materials_and_sections"),
        ("aasrp", "0005_insurance"),
    ]

    operations = [
        migrations.AlterField(
            model_name="aasrprequest",
            name="ship",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="srp_requests",
                to="eveuniverse.evetype",
            ),
        ),
        migrations.AlterField(
            model_name="aasrprequest",
            name="srp_link",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="srp_requests",
                to="aasrp.aasrplink",
            ),
        ),
        migrations.AlterField(
            model_name="aasrprequestcomment",
            name="srp_request",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="srp_request_comments",
                to="aasrp.aasrprequest",
            ),
        ),
    ]



"""
Preloads data required for this app from ESI
"""

# Standard Library
import logging

# Django
from django.core.management import call_command
from django.core.management.base import BaseCommand

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA SRP
from aasrp import __title__
from aasrp.constants import EVE_CATEGORY_ID_SHIP

logger = LoggerAddTag(logging.getLogger(__name__), __title__)


class Command(BaseCommand):
    help = "Preloads data required for this app from ESI"

    def handle(self, *args, **options):
        """
        Start the eve type import
        :param args:
        :param options:
        """

        call_command(
            "eveuniverse_load_types",
            __title__,
            "--category_id",
            str(EVE_CATEGORY_ID_SHIP),
        )
